/* $Id: e2_select_dir_dialog.c 354 2007-04-19 02:05:09Z tpgww $
 *
 * Copyright (C) 2005-2007 tooar <tooar@gmx.net>
 *
 * This file is part of emelFM2.
 * emelFM2 is free software; you can redistribute it and/or modify it
 * under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2, or (at your option)
 * any later version.
 *
 * emelFM2 is distributed in the hope that it will be useful, but
 * WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.
 * See the GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with emelFM2; see the file GPL. If not, contact the Free Software
 * Foundation, 51 Franklin Street, Fifth Floor, Boston, MA 02110-1301 USA
 */

#include "emelfm2.h"
#include "e2_dialog.h"

typedef struct _E2_SelDirData
{
	GtkFileChooser *chooser;
	gchar **choice;
	GtkWidget *togglebtn;
} E2_SelDirData;

//NOTE that toggling show-hidden via the widget's context menu
//cannot update the button icon ... should we abandon the icon ?
static void _e2_opendir_response_cb (GtkDialog *dialog, gint response,
	E2_SelDirData *data)
{
	gboolean nowhidden;
	gchar *dir;
	switch (response)
	{
		case E2_RESPONSE_USER1:
			//toggle hidden-items display & button-image
			nowhidden = gtk_file_chooser_get_show_hidden (data->chooser);
			gtk_file_chooser_set_show_hidden (data->chooser, ! nowhidden);
			e2_button_set_image (data->togglebtn, (nowhidden) ?
				"hidden_show_"E2IP".png" : "hidden_noshow_"E2IP".png");
			break;	//toggles handled by separate func
		case GTK_RESPONSE_OK:
			dir = gtk_file_chooser_get_current_folder (data->chooser);
			if (dir == NULL)
				*data->choice = NULL;
			else
			{
				gchar *utf = F_FILENAME_FROM_LOCALE (dir);	//not DISPLAY
				*data->choice = g_strconcat (utf, G_DIR_SEPARATOR_S, NULL);
				g_free (dir);
				F_FREE (utf);
			}
//		case GTK_RESPONSE_CANCEL:
		default:
			gtk_widget_destroy (GTK_WIDGET (dialog));
			gtk_main_quit ();
			break;
	}
}
/**
 * @brief display directory-chooser dialog
 *
 * @a newdir is set to a /-terminated string, newly-allocated,
 * or else NULL, if no or invalid choice is made
 *
 * @param currentdir utf string, path of dir to show in chooser when it opens
 * @param newdir pointer to store for chosen new dir-path
 *
 * @return
 */
void e2_opendir_dialog_create (gchar *currentdir, gchar **newdir)
{
	E2_SelDirData data;
	data.choice = newdir;
	GtkWidget *wchooser = gtk_file_chooser_widget_new
		(GTK_FILE_CHOOSER_ACTION_SELECT_FOLDER);
	data.chooser = GTK_FILE_CHOOSER (wchooser);
	GtkWidget *dialog = e2_dialog_create (NULL, NULL, _("open"),
		_e2_opendir_response_cb, &data);
	GtkWidget *vbox = GTK_DIALOG (dialog)->vbox;
	gtk_box_pack_start (GTK_BOX(vbox), wchooser, TRUE, TRUE, 0);

//	gtk_file_chooser_set_local_only (FALSE); FIXME vfs
/* trailing / ok
	if (g_str_has_suffix (currentdir, G_DIR_SEPARATOR_S)
	{
		gchar *p = currentdir + strlen (currendir);
		*(p-1) = '\0';
	} */
	gtk_file_chooser_set_current_folder (data.chooser, currentdir);

	data.togglebtn =
	e2_dialog_add_undefined_button_custom (dialog, FALSE, E2_RESPONSE_USER1,
		_("_Hidden"), "hidden_show_"E2IP".png",
		_("Toggle display of hidden directories"), NULL, NULL);
	e2_dialog_add_defined_button (dialog, &E2_BUTTON_CANCEL);
	e2_dialog_add_defined_button (dialog, &E2_BUTTON_OK);
	e2_dialog_setup (dialog, app.main_window);
//	gtk_widget_grab_focus (button); last one is already focused
	gint width, height;
	gtk_window_get_size (GTK_WINDOW (dialog), &width, &height);
	gtk_window_resize (GTK_WINDOW (dialog), width*2, height*6);

	e2_dialog_run (dialog, NULL, 0);
	gtk_main ();
}
