/* $Id: e2_fileview.h 372 2007-05-04 00:54:44Z tpgww $
 *
 * Copyright (C) 2004-2007 tooar <tooar@gmx.net>
 *
 * This file is part of emelFM2.
 * emelFM2 is free software; you can redistribute it and/or modify it
 * under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2, or (at your option)
 * any later version.
 *
 * emelFM2 is distributed in the hope that it will be useful, but
 * WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.
 * See the GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with emelFM2; see the file GPL. If not, contact the Free Software
 * Foundation, 51 Franklin Street, Fifth Floor, Boston, MA 02110-1301 USA
 */

#ifndef E2_FILEVIEW_H
#define E2_FILEVIEW_H

#include "emelfm2.h"

//type of "dir" in a pane
typedef enum
{
	FS_LOCAL   = 0,		//default, mounted-local dir
	FS_FUSE    = 1,		//virtual local dir that needs special handling
	FS_REMOTE  = 1 << 1,//virtual non-mounted remote dir
	FS_ARCHIVE = 1 << 2,//archive (can be local or remote)
} E2_FSType;
//types of virtual dir that are non-native, and warrant timeout if misbehaved
#define FS_SLOW (FS_FUSE | FS_REMOTE | FS_ARCHIVE)

#ifdef E2_VFS
//enumerator for type of backend to handle "dir" operations in a pane
typedef enum
{
	FS_NONE = -1,
	FS_SELF = 0,	//0 = default
	FS_FUSEAPI,	//only relevant for linux and freeBSD
	FS_MC,	//use lib from mc
	FS_XDG,	//use lib from freedesktop.org
	FS_GNOMEVFS,
	FS_GVFS,
	FS_MAX	//array-size counter (1 too big!)
} E2_FSHandler;
#endif

//these are in same order as ascii <, =, >
typedef enum { LT, EQ, GT } Operator;
typedef enum { E2_REFRESH, E2_RECREATE, E2_CHANGE } E2_RefreshType;
typedef enum { CD_NOTFINISHED, CD_TIMEOUT, CD_SUCCESS } E2_CDType;
//filelist treestore column-number enumerator
enum { FILENAME = 0, SIZE, PERM, OWNER, GROUP, MODIFIED, ACCESSED, CHANGED, MAX_COLUMNS };
//directory-read failure codes
enum
{
	E2DREAD_ENOENT,	//empty dir (not really an error)
	E2DREAD_NS,		//stat dir failed
	E2DREAD_DNR,	//dir not opened
	E2DREAD_DNCH,	//cannot cd into dir to poll its contents
	E2DREAD_DNF,	//read cancelled by user after timeout
	E2DREAD_ENOTH,	//failed to create thread when doing timed read
	E2DREAD_ENOMEM,	//not enough memory for storing data about read item
	E2DREAD_ELAST	//just an enumerator
};
#define E2DREAD_FAILED(ptr) (!(ptr == NULL || (gpointer)ptr > GUINT_TO_POINTER (E2DREAD_ENOMEM)))

typedef struct _E2_FileView
{
	GtkListStore *store;
	GtkTreeModel *model;	//may be a filter model or standard
	GtkWidget *treeview;
	GtkTreeSelection *selection;	//pointer to selection for this view's treeview
	GHashTable *selected_names;  //names of selected items (for reselection), or NULL
	gint sort_column;  //copy of current sort column, can be <0
	gboolean extsort;	//TRUE when name-sorting is by extension
	GtkSortType sort_order; //copy of current sort order
	GtkWidget *sort_arrows[MAX_COLUMNS];
	GtkLabel *name_label;	//for showing active pane by changing label content
	gchar dir[PATH_MAX];	//path shown in this view (utf-8)
	E2_FSType dirtype;	//flags for type of dir
#ifdef E2_VFS
	E2_FSHandler fs_type;	//enumerator of the vfs library/type for "dir" shown in this view (may be FS_SELF)
	gchar *tip;	/* for mounted dir NULL
	   for remote site "tip", utf8, no trailer, public components of 1st part of URI, before the filepath proper
	   for archive "tip", utf8, no trailer, pseudo-path of archive, with all ancestor(s) including remote URI and other archive
	*/
	gchar *priv_name;	/* for mounted dir NULL
	   for remote site excaped utf8, no trailer, login-ready URI before the filepath proper with p/w etc
	   for archive utf8, with trailer, path of temp dir used for unpacking whole or part
	*/
	gint archvfs_curr;	//index of app.vfs_data liststore row whose data reflect the archive last opened in this view
	gint remotevfs_curr;	//index of app.vfs_data liststore row whose data reflect the site last opened in this view
	GList *virt_history; //like dir_history, but specific to current vfs place
#endif
	GList *dir_history;  //list of structs with focused line etc for each opened local dir
//#ifndef E2_FAM
	time_t dir_mtime;
	time_t dir_ctime;
//#endif
	GtkWidget *check_dirs; //filter sub-menu toggle-widget
	GtkWidget *check_name; //filter sub-menu toggle-widget
	struct
	{
		gchar *patternptr;
		GSList *compiled_patterns;	//list of _E2_SelectPattern's
		gboolean invert_mask;
		gboolean case_sensitive;
		gboolean active;
	} name_filter;
	GtkWidget *check_size;
	struct
	{
		size_t size;
		Operator op;
		gboolean active;
	} size_filter;
	GtkWidget *check_date;
	struct
	{
		time_t time;	//time_t may be int or long int
		Operator op;
		enum {MTIME, ATIME, CTIME} time_type;
		gboolean active;
	} date_filter;

	gboolean filtered_before;	//state of prior call to filter function
	gboolean filter_directories;
	gboolean show_hidden;  //whether to display hidden items in this view
	guint total_items;	//total displayed and undisplayed items in the dir - 2 (to ignore . and ..)
	gint row;	//index of most-recently clicked row in the treeview
	gint drop_row;	//index of most-recently moused row during a drag operation
	gboolean lit;	//TRUE if drop-row is highlighted
	E2_RefreshType refreshtype; //whether a view update is a refresh, recreate or cd
	gboolean refresh_requested;	//TRUE when something has proposed that the view be refreshed
	gboolean blocked;	//flag to block view updates when refresh is disabled
	gboolean completed; //flag to block recursive view updates
	gchar last_find[NAME_MAX]; //buffer for storing names used in find task
//	GHookList hook_refresh;	//data for functions to run during post-refresh idle-time function
} FileView;

typedef struct _E2_FileInfo
{
	gchar filename[NAME_MAX+1];  //localised string
	struct stat statbuf;
} FileInfo;

typedef struct _E2_SelectedItemInfo
{
	gchar filename[NAME_MAX+1];  //localised string no trailing /
/*	FileInfo info;
	gchar *filename;  //selected item name (except no trailing /), utf-8 string
#ifdef E2_INCLIST
	//use this if the store data is changed on the fly
	GtkTreeRowReference *ref;
#else
	//this is used for un-selecting liststore items on the fly
	GtkTreePath *path;
#endif
*/
} E2_SelectedItemInfo;

typedef struct _E2_CDdata
{
	FileView *view;	//use this when re-starting a cd after timeout
	gchar *newpath;	//stored copy of place to open
#ifdef E2_VFSTMP
	gchar *prefix;
	E2_FSNewType fsclass;
#endif
	gboolean history;
	gboolean hook;
} E2_CDdata;

typedef struct _E2_CDwatch
{
	FileView *view;	//data for view being changed (can't use E2_PaneRuntime due to circular includes)
	gchar *newpath;	//place to be opened, utf8 with trailing separator
	gint repeats;	//current retry counter
//	guint watchtimer_id; //id of timer that is watching for completion
	E2_CDType *completed_flag;	//store for flag to set when completion detected
} E2_CDwatch;

typedef struct _E2_DRead
{
	pthread_t aid;	//read thread ID, 0 when stopped
	pthread_t mid;	//monitor thread ID, 0 when stopped
	GtkWidget *dialog;	//too-slow dialog widget, or NULL
} E2_DRead;

typedef struct _E2_SelectPattern
{
	GPatternSpec *pspec;
	gboolean negated;
} E2_SelectPattern;

gint e2_fileview_ext_sort (GtkTreeModel *model, GtkTreeIter *a, GtkTreeIter *b,
	GtkSortType *direction);
void e2_fileview_refilter_list (FileView *view);
void e2_fileview_filter_dirs_cb (GtkWidget *widget, FileView *view);
void e2_fileview_remove_filters_cb (GtkWidget *widget, FileView *view);
void e2_fileview_initialize_filters (FileView *view);

void e2_fileview_set_menu_position (GtkMenu *menu,
	gint *x, gint *y, gboolean *push_in, GtkWidget *treeview);
void e2_fileview_focus_row (FileView *view, gint row,
	gboolean select_row, gboolean clear_selection,
	gboolean center, gboolean grab_focus);
//void e2_fileview_select_row_by_filename (FileView *view, gchar *filename); unused
GHashTable *e2_fileview_log_selected_names (FileView *view);
void e2_fileview_reselect_names (FileView *view);
void e2_fileview_translate_cols_array (gint *src_array,
	gint *dest_array, gint size);
void e2_fileview_switch_views (void);
//void e2_fileview_switch_views_simple (void);
void e2_fileview_set_font (void);
void e2_fileview_set_row_background (FileView *view,
	GtkTreeIter *iter, GdkColor *color);
void e2_fileview_clear_row_background (FileView *view,
	GtkTreeIter *iter);
void e2_fileview_update_col_cachedata (void);
void e2_fileview_get_scroll_data (FileView *view, gint *leftcol, gint *toprow);
void e2_fileview_scroll_to_position (FileView *view, gint leftcol, gint toprow);
//gint e2_fileview_find_name_col (FileView *view); unused
gboolean e2_fileview_prepare_list (FileView *view);
void e2_fileview_scroll_list (GThread *viewthread);
void e2_fileview_refresh_list (FileView *view);
gboolean e2_fileview_treehash_free (GHashTable *data);
gboolean e2_fileview_cd_watch (E2_CDwatch *data);
gboolean e2_fileview_cd_manage (gpointer data);
//#ifdef E2_VFSTMPOK
//	E2_FSNewType fsclass, gchar *prefix,
//#endif
//	gchar *newpath, gboolean history, gboolean hook);
GtkWidget *e2_fileview_create_list (FileView *view);
void e2_fileview_select_all (GtkWidget *widget, FileView *view);
void e2_fileview_clean_selected (GPtrArray *selected);
GPtrArray *e2_fileview_get_selected (FileView *view);
GList *e2_fileview_get_selected_local (FileView *view);
FileInfo *e2_fileview_get_selected_first_local (FileView *view, gboolean updir);
gchar *e2_fileview_get_row_name (FileView *view, gint row);

#endif //ndef E2_FILEVIEW_H
